#include "job.h"
#include "master.h"

#define TILE_SIZE (32)

typedef struct {
   unsigned int next_tile_row;
   unsigned int next_tile_col;
   unsigned int tile_rows;
} Queue;

bool prepare_next_job(Job* job, Parameters* p, Queue* q) {
   if (q->next_tile_row >= q->tile_rows) return false;
   job->finished = false;
   job->row = q->next_tile_row * TILE_SIZE;
   job->column = q->next_tile_col * TILE_SIZE;
   if (job->column + TILE_SIZE >= p->width) {
      job->width = p->width - job->column;
      q->next_tile_row++; q->next_tile_col = 0;
   } else {
      job->width = TILE_SIZE;
      ++q->next_tile_col;
   }
   if (job->row + TILE_SIZE > p->height) {
      job->height = p->height - job->row;
   } else {
      job->height = TILE_SIZE;
   }
   return true;
}

bool master(struct shared_domain* sd, Parameters* p) {
   Queue queue = {
      .next_tile_row = 0,
      .next_tile_col = 0,
      .tile_rows = (p->height + (TILE_SIZE - 1)) / TILE_SIZE,
   };
   Job job;

   /* send out initial jobs */
   unsigned int nofprocesses = sd_get_nofprocesses(sd);
   unsigned int terminated = 0;
   for (unsigned int worker = 1; worker < nofprocesses; ++worker) {
      if (terminated || !prepare_next_job(&job, p, &queue)) {
	 job = (Job) { .finished = true };
	 ++terminated;
      }
      if (!sd_write(sd, worker, &job, sizeof job)) return false;
   }

   /* send out remaining jobs */
   if (!terminated) {
      while (prepare_next_job(&job, p, &queue)) {
	 unsigned int worker;
	 if (!sd_read(sd, &worker, sizeof worker)) return false;
	 if (!sd_write(sd, worker, &job, sizeof job)) return false;
      }
   }

   /* tell all active workers that there is no more work left */
   job = (Job) { .finished = true };
   for (unsigned int worker = nofprocesses - terminated - 1;
	 worker > 0; --worker) {
      if (!sd_write(sd, worker, &job, sizeof job)) return false;
   }
   return true;
}
