#include <gdk-pixbuf/gdk-pixbuf.h>
#include "job.h"
#include "mandelbrot.h"
#include "rgbcolor.h"
#include "worker.h"

static RGBColor get_color(Parameters* p, int row, int col) {
   double x = p->xmin + col/(double)p->resolution;
   double y = p->ymax - row/(double)p->resolution;
   return mandelbrot(x, y, p->maxiterations,
      p->colors, p->nof_colors,
      p->color_shift, p->color_progression);
}

bool worker(struct shared_domain* sd, Parameters* p, guchar* pixelbuf) {
   Job job;
   unsigned int rank = sd_get_rank(sd);
   while (sd_read(sd, &job, sizeof(Job)) && !job.finished) {
      /* process job */
      for (unsigned int row = job.row;
	    row < job.row + job.height; ++row) {
	 guchar* buf = pixelbuf + (row * p->width + job.column) * 3;
	 for (unsigned int col = job.column;
	       col < job.column + job.width; ++col) {
	    RGBColor color = get_color(p, row, col);
	    *buf++ = color.red * 255;
	    *buf++ = color.green * 255;
	    *buf++ = color.blue * 255;
	 }
      }
      /* signal that we need the next job */
      if (!sd_write(sd, 0, &rank, sizeof rank)) return false;
   }
   return true;
}
